#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
import ffgrid2
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm


####################################################
# from polar (lon, lat) to cartesian coords (x, y) #
####################################################

def new_cartesian_grid(nb_days, jour, month, longi, lati, z_ini, z0, z1, dx, dy):


    ############################
    # definition of input data #
    ############################
    # jour = vector of days (1D-array)
    # month = number or days in month (integer)
    # longi = longitude of data points (1D-array)
    # lati = latitude of data points (1D-array)
    # z_ini = data to re-grid (1D-array)
    # z0 = min value of data (float)
    # z1 = max value of data (float)


    ###############################################
    # definition of the new cartesian grid (x, y) #
    ############################################### 
    x0 = -3000. # min limit of grid
    x1 = 2500. # max limit of grid
    xvec = np.arange(x0, x1+dx, dx)
    nx = len(xvec) 
    y0 = -3000. # min limit of grid
    y1 = 3000. # max limit of grid
    yvec = np.arange(y0, y1+dy, dy)
    ny = len(yvec)
    xgrid_cart, ygrid_cart= np.meshgrid(xvec, yvec) # new cartesian grid (centered on North pole)


    ###################################
    # calculation of the new gridding #
    ################################### 
    zgrid_output = np.zeros([ny, nx, nb_days], float)
    ngrid_output = np.zeros([ny, nx, nb_days], float)
    z2grid_output = np.zeros([ny, nx, nb_days], float)
    sigmagrid_output = np.zeros([ny, nx, nb_days], float)
    bblat = nonzero(lati >= 65.) # only select high latitude values of z
    for ijr in range (0, nb_days): # loop on time - here time = days
        bbjour = nonzero(jour[bblat] == ijr + 1.)[0]
        longitude = longi[bblat][bbjour]
        latitude = lati[bblat][bbjour]
        z = z_ini[bblat][bbjour]    
        #################################################################################
        # associates a (x, y) couple to each point of (longitude, latitude) coordinates #
        #################################################################################
        # origin of the (x, y) grid : (x=0, y=0) <=> (lon=0, lat=0) 
        L = len(z)
        Rt = 6371.
        alpha = (pi*Rt)/180.
        beta = pi/180.
        x = np.zeros([L], float)
        y = np.zeros([L], float)
        for k in range (0, L):
                if ((longitude[k] >= 0.) & (longitude[k] <= 90.)): # 4eme quadrant
                        theta = (90. - longitude[k]) * beta
                        x[k] = (90. - latitude[k]) * alpha * cos(theta)
                        y[k] = (90. - latitude[k]) * alpha * sin(-theta)
                else:
                        if ((longitude[k] > 90.) & (longitude[k] <= 180.)): # 1er quadrant
                                theta = (longitude[k] - 90.) * beta
                                x[k] = (90. - latitude[k]) * alpha * cos(theta)
                                y[k] = (90. - latitude[k]) * alpha * sin(theta)
                        else: 
                                if ((longitude[k] >= -180.) & (longitude[k] < 0.)): # 2eme et 3eme quadrants
                                        theta = (270. + longitude[k]) * beta
                                        x[k] = (90. - latitude[k]) * alpha * cos(theta)
                                        y[k] = (90. - latitude[k]) * alpha * sin(theta)
        #################################################
        # counting each x and y value in new grid cells #
        #################################################
        ix = np.zeros([L], int)
        i = 0
        for i in range (0, L): # boucle sur les points M (abscisses) 
                if x[i] == x0:
                        ix[i] = 0
                else:
                        ix[i] = math.ceil((x[i] - x0) / dx) - 1 # associates each x vakue to a cell number
        i = 0
        iy = np.zeros([L], int) 
        for i in range (0, L):# boucle sur les points M (ordonnees) 
                if y[i] == y0:
                        iy[i] = 0
                else:
                        iy[i] = math.ceil((y[i] - y0) / dy) - 1 # associates each y vakue to a cell number
        #########################################################################################
        # calculation of distances between point M(x,y) and 4 grid points of its belonging cell #
        #########################################################################################
        close_point = np.zeros([L, 2], int)
        k = 0
        for k in range (0, L): # boucle sur les points M (x et y)
                d1 = sqrt(((x[k] - xvec[ix[k]]) ** 2) + ((y[k] - yvec[iy[k]]) ** 2))
                d2 = sqrt(((x[k] - xvec[ix[k] + 1]) ** 2) + ((y[k] - yvec[iy[k]]) ** 2))
                d3 = sqrt(((x[k] - xvec[ix[k]]) ** 2) + ((y[k] - yvec[iy[k] + 1]) ** 2))
                d4 = sqrt(((x[k] - xvec[ix[k] + 1]) ** 2) + ((y[k] - yvec[iy[k] + 1]) ** 2))
                d_vec = np.array([d1, d2, d3, d4])
                ind = np.where(d_vec == min(d_vec)) # finds the smallest distance between the 4 points of the grid
                point_vec = np.array([(ix[k], iy[k]), (ix[k] + 1, iy[k]), (ix[k], iy[k] + 1), (ix[k] + 1, iy[k] + 1)]) 
                close_point[k, :] = point_vec[ind[0][0]]# we have chosen which point of the grid is closer to M // point_vec[ind[0][0]] = (cell no of closest x, celle no of closest y)
        ################################################
        # association of z value to closest grid point #
        ################################################
        zgrid = np.zeros([ny, nx], float) # z in new grid
        ngrid = np.zeros([ny, nx], int) # nb of obs per new grid cell
        z2grid = np.zeros([ny, nx], float) # z**2 in new grid
        for k in range (0, L):
                zgrid[close_point[k, 1], close_point[k, 0]] = zgrid[close_point[k, 1], close_point[k, 0]] + z[k]
                ngrid[close_point[k, 1], close_point[k, 0]] = ngrid[close_point[k, 1], close_point[k, 0]] + 1
                z2grid[close_point[k, 1], close_point[k, 0]] = z2grid[close_point[k, 1], close_point[k, 0]] + (z[k] * z[k])
        ZGRID = zgrid / ngrid
        ##############################
        # variance in each grid cell #
        ##############################
        sigmagrid = np.zeros([ny, nx], float)
        for j in range (0, nx):
                for i in range (0, ny):
                        if (ngrid[i, j] > 1): # take away the cells where no or one obs 
                                sigmagrid[i, j] = sqrt((1./(ngrid[i, j]-1.))*(z2grid[i, j] - ngrid[i, j]*ZGRID[i, j]*ZGRID[i, j]))
                        else:
                                if (ngrid[i, j] == 1):
                                        sigmagrid[i, j] = 0.
                                else:
                                        sigmagrid[i, j] = NaN

        zgrid_output[:, :, ijr] = ZGRID[:, :]
        ngrid_output[:, :, ijr] = ngrid[:, :]
        z2grid_output[:, :, ijr] = z2grid[:, :]
        sigmagrid_output[:, :, ijr] = sigmagrid[:, :]


    return zgrid_output, ngrid_output, z2grid_output, sigmagrid_output, xvec, yvec, xgrid_cart, ygrid_cart 





