#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm
from netCDF4 import Dataset
import arctic_map # function to regrid coast limits
import cartesian_grid_test # function to convert grid from polar to cartesian
import scipy.special
import ffgrid2
import map_ffgrid



#####################
# read fichier .DAT #
#####################
fichier=open('/mma/hermozol/Documents/Data_tests/GLACE/GLACE_AMSUB89_EMIS_SEPTEMBER2009.DAT','r')
numlines = 0
for line in fichier: numlines += 1

fichier.close

fichier=open('/mma/hermozol/Documents/Data_tests/GLACE/GLACE_AMSUB89_EMIS_SEPTEMBER2009.DAT','r')

nbtotal=numlines-1

iligne=0
lat=np.zeros([nbtotal],float)
lon=np.zeros([nbtotal],float)
jjr=np.zeros([nbtotal],float)
zen=np.zeros([nbtotal],float)
fov=np.zeros([nbtotal],float)
ts=np.zeros([nbtotal],float)
emis=np.zeros([nbtotal],float)
tb=np.zeros([nbtotal],float)
tup=np.zeros([nbtotal],float)
tdn=np.zeros([nbtotal],float)
trans=np.zeros([nbtotal],float)

# format:
# printf, luna2, xlon, ylat, annee, mois, jour, heure, jour_obs, heure_obs, tsy, zeny, fovy, saty, masque, ee89, tb89, tup89, tdn89, trans89, format='(18(f12.3,1X))'  

while (iligne < nbtotal) :
         line=fichier.readline()
         # exemple : line = "0.22 2.3 5.0 6"
         liste = line.split()
         # exemple : listeCoord ['0.22', '2.3', '5.0', '6'] (liste de chaine de caract?es)
         lat[iligne] = float(liste[1])
         lon[iligne] = float(liste[0])
         jjr[iligne] = float(liste[4])
         ts[iligne] = float(liste[8])
         fov[iligne] = float(liste[10])
         zen[iligne] = float(liste[9])
         emis[iligne] = float(liste[13])
         tb[iligne] = float(liste[14])
         tup[iligne] = float(liste[15])
         tdn[iligne] = float(liste[16])
         trans[iligne]  = float(liste[17])
         iligne=iligne+1
         

fichier.close()

for i in range (0, nbtotal):
    if (emis[i] < 0.):
        emis[i] = NaN 
    if (emis[i] > 1.):
        emis[i] = 0.99  



############################
# definition of parameters #
############################
bblat = nonzero(lat >= 65.) # only high latitudes
llon = lon[bblat] # longitude
llat = lat[bblat] # latitude
zen_angle = zen[bblat]# zenith angle
field_view = fov[bblat] # field of view
e = emis[bblat] # emissivity
t_surf = ts[bblat] # surface temperature
t_up = tup[bblat] # tup
t_dn = tdn[bblat] # tdown
transm = trans[bblat] # transmissivity
t_b = tb[bblat] # brightness temperature
L = shape(bblat)[1]


#######################################
# calculation of lamb_spec parameters #
#######################################
r = pi / 180.
opac = - cos(zen_angle * r) * np.log(transm) 
E3 = scipy.special.expn(3, opac)
theta_eff = np.zeros([L], float)
tdn_lamb = np.zeros([L], float)
tb_spec = np.zeros([L], float)
tb_lamb = np.zeros([L], float)
e_spec = np.zeros([L], float)
e_lamb = np.zeros([L], float)
e_spec_lamb = np.zeros([L], float)
for ii in range (0, L):
    theta_eff[ii] = math.acos(- opac[ii] / (np.log(2. * E3[ii]))) * (1. / r)
    tdn_lamb[ii] = t_surf[ii] * (1 - exp(- opac[ii] / cos (theta_eff[ii] * r))) # Tdown to calculate with theta_eff
    tb_spec[ii] = (t_surf[ii] * e[ii] * exp(- opac[ii])) + (1. - e[ii]) * exp(- opac[ii]) * t_dn[ii] + t_up[ii] # calculation of Tb spec with theta
    tb_lamb[ii] = (t_surf[ii] * e[ii] * exp(- opac[ii])) + (1. - e[ii]) * exp(- opac[ii]) * tdn_lamb[ii] + t_up[ii] # calculation of Tb spec with theta_eff (with Tdown_lamb)
    e_spec[ii] = ((tb_spec[ii] - t_up[ii]) * exp(opac[ii]) - t_dn[ii]) / (t_surf[ii] - t_dn[ii])
    e_lamb[ii] = ((tb_lamb[ii] - t_up[ii]) * exp(opac[ii]) - t_dn[ii]) / (t_surf[ii] - t_dn[ii]) 
    e_spec_lamb[ii] = e_lamb[ii] - e_spec[ii]



plt.ion()
plt.figure()
plt.scatter(opac, theta_eff)
xlim(0.14, 0.42)
ylim(52.9, 56.2)
xticks(np.arange(0.14, 0.44, 0.02))
grid(True)
xlabel('zenith opacity')
ylabel('effective incidence angle (deg)')
title('SEPTEMBER 2009 - AMSUB')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/scatter_theta_eff-opacity_sept2009_AMSUB.png')


plt.figure()
plt.scatter(field_view, e_spec_lamb)
xlim(0., 90.)
xticks(np.arange(0, 90, 5))
grid(True)
xlabel('scan position')
ylabel('emissivity SPEC-LAMB')
title('SEPTEMBER 2009 - AMSUB')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/scatter_e-spec_e-lamb_fov_sept2009_AMSUB.png')



e_diff_mean = np.zeros([89], float)
e_diff_ect = np.zeros([89], float)
for ifov in range (0, 89):
    ind = np.where(field_view == float(ifov))[0]
    if (ind == []):
        e_diff_mean[ifov] = NaN
        e_diff_ect[ifov] = NaN
    else:
        e_diff_mean[ifov] = mean(e_spec_lamb[ind])
        e_diff_ect[ifov] = sqrt(sum((e_spec_lamb[ind] - e_diff_mean[ifov]) ** 2.))
    if (e_diff_ect[ifov] == 0.):
        e_diff_ect[ifov] = NaN


    

plt.figure()
plt.plot(e_diff_mean, 'bo', label = 'mean value')
xlabel('field of view')
ylabel('emissivity SPEC-LAMB')
legend(loc = 2)
twinx()
plot(e_diff_ect , 'go', label = 'std value')
xlabel('standard deviation')
xticks(np.arange(0, 90, 10))
legend(loc = 1)
plt.title('SEPTEMBER 2009 - AMSUB')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/evo_emis_lamb-spec_fov_sept2009_AMSUB.png')





##############################################
# stack of lamb_spec parameters in .txt file #
##############################################
data_spec_lamb = open ('/mma/hermozol/Documents/Data_tests/monthly_GLACE/lamb_spec_param_all_angles_SEPTEMBER2009.dat', 'a')
for ii in range (0, L):
    data_spec_lamb.write(('%(lon)4.2f    %(lat)4.2f    %(za)4.2f    %(fov)2.0f   %(emis)5.4f   %(tsurf)5.2f   %(tup)5.2f   %(tdn)5.2f    %(trans)5.4f   %(tb)5.2f    %(opac)5.4f    %(theta_eff)5.3f    %(tdn_lamb)5.2f    %(tb_spec)5.2f    %(tb_lamb)5.2f    %(e_spec)5.4f    %(e_lamb)5.4f    %(e_diff)5.4f \n' % {
                                            'lon':llon[ii],
                                            'lat':llat[ii],
                                            'za':zen_angle[ii],
                                            'fov':field_view[ii],
                                            'emis':e[ii],
                                            'tsurf':t_surf[ii],
                                            'tup':t_up[ii],
                                            'tdn':t_dn[ii],
                                            'trans':transm[ii],
                                            'tb':t_b[ii],
                                            'opac':opac[ii],
                                            'theta_eff':theta_eff[ii],
                                            'tdn_lamb':tdn_lamb[ii],
                                            'tb_spec':tb_spec[ii],
                                            'tb_lamb':tb_lamb[ii],
                                            'e_spec':e_spec[ii],
                                            'e_lamb':e_lamb[ii],
                                            'e_diff':e_spec_lamb[ii],
                                            }))


data_spec_lamb.close()




