#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm
from netCDF4 import Dataset
import arctic_map # function to regrid coast limits
import cartesian_grid_test # function to convert grid from polar to cartesian
import scipy.special
import ffgrid2
import map_ffgrid
from matplotlib import colors
from matplotlib.font_manager import FontProperties
import map_cartesian_grid

MONTH = np.array(['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'])
month = np.array(['JANUARY', 'FEBRUARY', 'MARCH', 'APRIL', 'MAY', 'JUNE', 'JULY', 'AUGUST', 'SEPTEMBER', 'OCTOBER', 'NOVEMBER', 'DECEMBER'])
month_day = np.array([31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31])
M = len(month)


########################
# grid characteristics #
########################
x0 = -3000. # min limit of grid
x1 = 2500. # max limit of grid
dx = 40.
xvec = np.arange(x0, x1+dx, dx)
nx = len(xvec) 
y0 = -3000. # min limit of grid
y1 = 3000. # max limit of grid
dy = 40.
yvec = np.arange(y0, y1+dy, dy)
ny = len(yvec)




#########################################
# comparison with OSISAF sea ice extent #
#########################################
osi_ice = np.zeros([M, 31, 151, 139])
for imo in range (0, M):
    print 'open OSISAF file month ' + str(month[imo])
    fichier_osisaf = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/OSISAF_Ice_Types/OSISAF_ice_types_cartesian_grid/OSISAF_ice_types_cartes_grid_' + str(month[imo]) + '_2009.nc', 'r', format='NETCDF3_CLASSIC')
    xdist = fichier_osisaf.variables['longitude'][:]
    ydist = fichier_osisaf.variables['latitude'][:]
    day = fichier_osisaf.variables['days'][:]
    osi_type = fichier_osisaf.variables['osi_ice_type'][:]
    fichier_osisaf.close()
    for ilon in range (0, len(xdist)):
        for ilat in range (0, len(ydist)):
            for ijr in range (0, month_day[imo]):
                if ((xdist[ilon] >= -312.5) & (xdist[ilon] <= 328.5) & (ydist[ilat] >=-334.3) & (ydist[ilat] <= 338.5)):
                    osi_ice[imo, ijr, ilat, ilon] = 1.
                else:
                    if (osi_type[ijr, ilat, ilon] >= 2.):
                        osi_ice[imo, ijr, ilat, ilon] = 1.
                    else:
                        if (isnan(osi_type[ijr, ilat, ilon]) == True):
                            osi_ice[imo, ijr, ilat, ilon] = NaN
                        else: 
                            osi_ice[imo, ijr, ilat, ilon] = 0.


# test:
x_ind, y_ind, z_ind, volume = arctic_map.arctic_map_lat50()
x_coast = x_ind
y_coast = y_ind
z_coast = z_ind
colormap = colors.ListedColormap(['0.5', 'cyan','blue', 'green'])
map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, osi_type[10, :, :], 0, 5, 1, colormap, 'ice_type')
map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, osi_ice[3, 7, :, :], -1., 2, 1, cm.s3pcpn_l_r, 'test')







##################################
# calculation of OSISAF ice area #
##################################
pix_area = 40. * 40.
nb_pix_osisaf = np.zeros([M, 31], float)
total_ice_area_osisaf = np.zeros([M, 31], float)
total_monthly_ice_area_osisaf = np.zeros([M], float)
std_total_monthly_ice_area_osisaf = np.zeros([M], float)
for imo in range (0, M):
    for ijr in range (0, month_day[imo]):
        ice = reshape(osi_ice[imo, ijr, :, :], size(osi_ice[imo, ijr, :, :]))
        nb_pix_osisaf[imo, ijr] = len(np.where(ice == 1.)[0])
        total_ice_area_osisaf[imo, ijr] = (pix_area * nb_pix_osisaf[imo, ijr]) # daily total ice area
    total_monthly_ice_area_osisaf[imo] = mean(total_ice_area_osisaf[imo, 0 : month_day[imo]][nonzero(isnan(total_ice_area_osisaf[imo, 0 : month_day[imo]]) == False)]) # monthly mean total ice area
    alpha = 1. / float(month_day[imo] - 1)
    area_mean = mean(total_ice_area_osisaf[imo, :][nonzero(isnan(total_ice_area_osisaf[imo, :]) == False)])
    std_sum = sum((total_ice_area_osisaf[imo, :] - area_mean) ** 2)
    std_total_monthly_ice_area_osisaf[imo] = sqrt(alpha * std_sum)








#################################
# stack of ice area OSISAF data #
#################################

#### total ice area in .DAT file ####
print 'start stacking in .dat file'
data_osisaf = open ('/net/argos/data/parvati/lahlod/ARCTIC/OSISAF_Ice_Types/OSISAF_daily_ice_no-ice_2009.dat', 'a')
for imo in range (0, M):
    for ijr in range (0, month_day[imo]):
        data_osisaf.write(('%(months)10s    %(days)2.0f    %(total_monthly_ice_area_osisaf)10.5f    %(std_total_monthly_ice_area_osisaf)10.5f    %(total_ice_area_osisaf)10.5f   \n' % {
                                            'months':month[imo],
                                            'days':np.arange(1, month_day[imo] + 1)[ijr],
                                            'total_monthly_ice_area_osisaf':total_monthly_ice_area_osisaf[imo],
                                            'std_total_monthly_ice_area_osisaf':std_total_monthly_ice_area_osisaf[imo],
                                            'total_ice_area_osisaf':total_ice_area_osisaf[imo, ijr],
                                            }))

data_osisaf.close()


#### OSISAF sea ice extent maps in NETCDF monthly files ####
print 'stacking of gridded data'
for imo in range (0, M):
    print 'file for month ' + str(month[imo])
    rootgrp = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/OSISAF_Ice_Types/OSISAF_ice_types_cartesian_grid/OSISAF_daily_ice_no-ice_' + month[imo] + '2009.nc', 'w', format='NETCDF3_CLASSIC')
    rootgrp.createDimension('longitude', nx)
    rootgrp.createDimension('latitude', ny)
    rootgrp.createDimension('days', month_day[imo])
    nc_x = rootgrp.createVariable('longitude', 'f', ('longitude',))
    nc_y = rootgrp.createVariable('latitude', 'f', ('latitude',))
    nc_day = rootgrp.createVariable('days', 'f', ('days',))
    nc_ice = rootgrp.createVariable('ice_osi', 'f', ('days', 'latitude', 'longitude'))
    nc_x[:] = xvec
    nc_y[:] = yvec
    nc_day[:] = np.arange(0, month_day[imo], 1)
    nc_ice[:] = osi_ice[imo, 0 : month_day[imo], :, :] # = 0 if open sea // = 1 if sea ice // = NaN if land 
    rootgrp.close()
