#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm
from netCDF4 import Dataset
import arctic_map # function to regrid coast limits
import cartesian_grid_test # function to convert grid from polar to cartesian
import scipy.special
import ffgrid2
import map_ffgrid





dx=1.
dy=0.5
x0, x1 = -180., 180.
y0, y1 = 65., 90.
xvec = np.arange(x0, x1 + dx, dx)
yvec = np.arange(y0, y1 + dy, dy)
nx = len(xvec)
ny = len(yvec)
r = pi /180. 
nx = len(lon_grid)
ny = len(lat_grid)
#plt.ion()
#map_ffgrid.draw_map_l (xvec, yvec, 0.5, 0.99, 0.01, opac_month, 'emissivity')






###################################
# mixed spec and lamb assumptions #
###################################
s = np.array([0., 0.25, 0.5, 0.75, 1.]) # specularity coefficient (s=1 <=> specular)
S = len(s)
new_tdn_month = np.zeros([ny, nx, S], float)
new_emis_month = np.zeros([ny, nx, S], float)
for s_ind in range (0, S):
    for ilon in range (0, nx):
        for ilat in range (0, ny):
            new_tdn_month[ilat, ilon, s_ind] = s[s_ind] * tdn_month[ilat, ilon] + (1. - s[s_ind]) * td_lamb_month[ilat, ilon]
            new_emis_month[ilat, ilon, s_ind] = ((tb_month[ilat, ilon] - tup_month[ilat, ilon]) * exp(opac_month[ilat, ilon]) - new_tdn_month[ilat, ilon, s_ind]) / (tsurf_month[ilat, ilon] - new_tdn_month[ilat, ilon, s_ind])


# scatter plots new_emis function of emissivity
plt.figure()
color = np.array(['b', 'r', 'g', 'c', 'y'])
corr = np.zeros([S], float)
for s_ind in range (0, S):
    emis1 = reshape(new_emis_month[:, :, s_ind], size(new_emis_month[:, :, s_ind]))
    emis0 = reshape(emis_month, size(emis_month))
    corr[s_ind] = np.corrcoef(emis1[nonzero(isnan(emis1) == False)], emis0[nonzero(isnan(emis0) == False)])[0][1]
    plt.scatter(emis1, emis0, c = color[s_ind], alpha = 0.6, label = 's = ' + str(s[s_ind]) + '-corr = ' + str(corr[s_ind]))

plt.plot(np.arange(0.4, 1.1, 0.1), np.arange(0.4, 1.1, 0.1), 'k')
xlim(0.4, 1.)
ylim(0.4, 1.)
legend(loc = 4)
xlabel('new_emissivity')
ylabel('emissivity')
title('AMSUB - 1sept2009')
plt.savefig('/mma/hermozol/Documents/figure_output/spec_coef_evolution/scatter_new_emiss_emis_amsub_1sept2009.png')



###############################
# mapping of different params #
###############################
# definition of grid parameters
x = lon_grid
y = lat_grid
dx=1.
dy=0.5
x0, x1 = -180., 180.
y0, y1 = 65., 90.
xvec = np.arange(x0, x1 + dx, dx)
yvec = np.arange(y0, y1 + dy, dy)
nx = len(xvec)
ny = len(yvec)

# emissivity
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 0.4, 0.99, 0.01, emis_month, 'emissivity')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_emis_AMSUB_sept2009.png')
# emissivity, lamb assumption
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 0.4, 0.99, 0.01, e_lamb_month, 'emis, lamb surf')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_emis_lamb_AMSUB_sept2009.png')
# emissivity, spec assumption
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 0.4, 0.99, 0.01, e_spec_month, 'emis, spec surf')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_emis_spec_AMSUB_sept2009.png')
# emissivity difference lamb-spec
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 0.007, 0.06, 0.0001, e_diff_month, 'emis difference lamb-spec' )
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_emis_diff_lamb-spec_lamb_AMSUB_sept2009.png')
# brightness temp
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 190., 257., 1., tb_month, 'Tb')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_tb_AMSUB_sept2009.png')
# brightness temp, lamb assumption
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 198., 262., 1., tb_lamb_month, 'Tb, lamb surf')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_tb_lamb_AMSUB_sept2009.png')
# brightness temp, spec assumption
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 193., 260., 1., tb_spec_month, 'Tb, spec surf')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_tb_spec_AMSUB_sept2009.png')
# Tdown 
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 22., 65., 1., tdn_month, 'Td')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_td_AMSUB_sept2009.png')
# Tdown lamb
plt.ion()
map_ffgrid.draw_map_l (xvec, yvec, 39., 100., 1., td_lamb_month, 'Td, lamb surf')
plt.savefig('/mma/hermozol/Documents/figure_output/comp_lamb_spec/monthly_td_lamb_AMSUB_sept2009.png')

# new emissivity with specularity coefficient s
for s_ind in range (0, S): # loop on s coefficient (s = 0 --> lamb surface // s = 1 --> spec surf)
    map_ffgrid.draw_map_l (xvec, yvec, 0.4, 0.99, 0.01, new_emis_month[:, :, s_ind], 'new emiss specularity = ' + str(s[s_ind]) + ' AMSUB - sept2009')
    plt.savefig('/mma/hermozol/Documents/figure_output/spec_coef_evolution/monthly_new_emis_s-' + str(s[s_ind]) + '_AMSUB_sept2009.png')


map_ffgrid.draw_map_l (xvec, yvec, 0.5, 0.99, 0.01, new_emis_month[:, :, 1], 'emissivity')

