#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm
from netCDF4 import Dataset
import arctic_map # function to regrid coast limits
import cartesian_grid_test # function to convert grid from polar to cartesian
import scipy.special
import ffgrid2
import map_ffgrid
from matplotlib import colors
from matplotlib.font_manager import FontProperties
import map_cartesian_grid


###############################
# time period characteristics #
###############################
MONTH = np.array(['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'])
month = np.array(['JANUARY', 'FEBRUARY', 'MARCH', 'APRIL', 'MAY', 'JUNE', 'JULY', 'AUGUST', 'SEPTEMBER', 'OCTOBER', 'NOVEMBER', 'DECEMBER'])
month_day = np.array([31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31])
M = len(month)


########################
# grid characteristics #
########################
x0 = -3000. # min limit of grid
x1 = 2500. # max limit of grid
dx = 40.
xvec = np.arange(x0, x1+dx, dx)
nx = len(xvec) 
y0 = -3000. # min limit of grid
y1 = 3000. # max limit of grid
dy = 40.
yvec = np.arange(y0, y1+dy, dy)
ny = len(yvec)




###################################

frequ = np.array(['23', '30', '50', '89'])
ice_spec = np.zeros([4, M, ny, nx], float)
ice_lamb = np.zeros([4, M, ny, nx], float)
ice_diff = np.zeros([4, M, ny, nx], float)
ice_ratio = np.zeros([4, M, ny, nx], float)
ice_spec_anom = np.zeros([4, M, ny, nx], float)
ice_lamb_anom = np.zeros([4, M, ny, nx], float)
ice_diff_anom = np.zeros([4, M, ny, nx], float)
ice_ratio_anom = np.zeros([4, M, ny, nx], float)
for ifr in range (0, 4):
    print 'frequency ' + frequ[ifr]
    for imo in range (0, M):
        print 'month ' + month[imo]
        fichier_ice = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/AMSUA_ice_class/sub_classification/cartesian_grid_map_sea_ice_extent_with-AMSUA23-and-30_' + month[imo] + '2009_AMSUA' + str(frequ[ifr]) + '_spec_lamb_thresholds.nc', 'r', format='NETCDF3_CLASSIC')
        ice_spec[ifr, imo, :, :] = fichier_ice.variables['spec_ice_area'][:]
        ice_lamb[ifr, imo, :, :] = fichier_ice.variables['lamb_ice_area'][:]
        ice_diff[ifr, imo, :, :] = fichier_ice.variables['diff_ice_area'][:]
        ice_ratio[ifr, imo, :, :] = fichier_ice.variables['ratio_ice_area'][:]
        fichier_ice.close()
        a = reshape(ice_spec[ifr, imo, :, :], size(ice_spec[ifr, imo, :, :]))
        b = reshape(ice_lamb[ifr, imo, :, :], size(ice_lamb[ifr, imo, :, :]))
        c = reshape(ice_diff[ifr, imo, :, :], size(ice_diff[ifr, imo, :, :]))
        d = reshape(ice_ratio[ifr, imo, :, :], size(ice_ratio[ifr, imo, :, :]))
        print 'calculate anomaly'
        for ilat in range (0, ny):
            for ilon in range (0, nx):
                ice_spec_anom[ifr, imo, ilat, ilon] = ice_spec[ifr, imo, ilat, ilon] - mean(a[nonzero(isnan(a) == False)])
                ice_lamb_anom[ifr, imo, ilat, ilon] = ice_lamb[ifr, imo, ilat, ilon] - mean(b[nonzero(isnan(b) == False)])
                ice_diff_anom[ifr, imo, ilat, ilon] = ice_diff[ifr, imo, ilat, ilon] - mean(c[nonzero(isnan(c) == False)])
                ice_ratio_anom[ifr, imo, ilat, ilon] = ice_ratio[ifr, imo, ilat, ilon] - mean(d[nonzero(isnan(d) == False)])
        ########################
        # stack in netcdf file #
        ######################## 
        print 'stack in file month ' + str(month[imo])
        rootgrp = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/AMSUA_ice_class/sub_classification/cartesian_grid_map_sea_ice_extent_with-AMSUA23-and-30_param_anomaly_' + month[imo] + '2009_AMSUA' + str(frequ[ifr]) + '.nc', 'w', format='NETCDF3_CLASSIC')
        rootgrp.createDimension('longitude', len(xvec))
        rootgrp.createDimension('latitude', len(yvec))
        nc_lon = rootgrp.createVariable('longitude', 'f', ('longitude',))
        nc_lat = rootgrp.createVariable('latitude', 'f', ('latitude',))
        nc_anom_spec = rootgrp.createVariable('spec_anomaly', 'f', ('latitude', 'longitude'))
        nc_anom_lamb = rootgrp.createVariable('lamb_anomay', 'f', ('latitude', 'longitude'))
        nc_anom_diff = rootgrp.createVariable('diff_anomaly', 'f', ('latitude', 'longitude'))
        nc_anom_ratio = rootgrp.createVariable('ratio_anomaly', 'f', ('latitude', 'longitude'))
        nc_lon[:] = xvec
        nc_lat[:] = yvec
        nc_anom_spec[:] = ice_spec_anom[ifr, imo, :, :]
        nc_anom_lamb[:] = ice_lamb_anom[ifr, imo, :, :]
        nc_anom_diff[:] = ice_diff_anom[ifr, imo, :, :]
        nc_anom_ratio[:] = ice_ratio_anom[ifr, imo, :, :]
        rootgrp.close()



'''

# test:
ion()
x_ind, y_ind, z_ind, volume = arctic_map.arctic_map_lat50()
x_coast = x_ind
y_coast = y_ind
z_coast = z_ind
for ifr in range (0, 2):
    for imo in range (0, M):
        map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, ice_diff_anom[ifr, imo, :, :], ice_diff_anom[nonzero(isnan(ice_diff_anom) == False)].min(), ice_diff_anom[nonzero(isnan(ice_diff_anom) == False)].max(), 0.0001, cm.s3pcpn_l_r, 'Emissivity diff anomaly')
        title('AMSUA ' + str(frequ) + ' - ' + str(month[imo]) + ' 2009')
        plt.savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/ice_class_AMSUA/sub_classification/maps_sea_ice_extent/emiss_diff_anomaly_map_AMSUA'+str(frequ[ifr])+'_with_AMSUA23-and-30-spec_'+str(month[imo])+'_2009.png')
#cm.s3pcpn_l_r

'''




'''
frequ = 89
spec_ice = np.zeros([M, ny, nx], float)
lamb_ice = np.zeros([M, ny, nx], float)
diff_ice = np.zeros([M, ny, nx], float)
spec_month = np.zeros([M, ny, nx], float)
lamb_month = np.zeros([M, ny, nx], float)
diff_month = np.zeros([M, ny, nx], float)
std_spec = np.zeros([M, ny, nx], float)
std_lamb = np.zeros([M, ny, nx], float)
std_diff = np.zeros([M, ny, nx], float)
for imo in range (0, 1):
    print 'month ' + month[imo]
    ##################################################################################
    # choice of AMSUA 23GHz delimitation ice/no_ice for the sub_classification study #
    ##################################################################################
    print 'open threshold file'
    fichier_emis = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/AMSUA_ice_class/cartesian_grid_map_ice_no-ice_' + str(month[imo]) + '2009_AMSUA23_spec_lamb_thresholds.nc', 'r', format='NETCDF3_CLASSIC')
    spec_lim = fichier_emis.variables['spec_ice_area'][:]
    fichier_emis.close()
    fichier_e = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/monthly_GLACE/gridded_data/cartesian_grid/res_40/cartesian_grid_monthly_data_lamb_spec_near_nadir_AMSUA' + str(frequ) + '_' + str(month[imo]) + '2009.nc', 'r', format='NETCDF3_CLASSIC')
    day = fichier_e.variables['days'][:]
    emis_spec = fichier_e.variables['e_spec'][:]
    emis_lamb = fichier_e.variables['e_lamb'][:]
    emis_diff = fichier_e.variables['e_spec_lamb'][:]
    fichier_e.close()
    for ilon in range (0, nx):
        for ilat in range (0, ny):
           if (isnan(spec_lim[ilat, ilon]) == True):
                spec_ice[imo, ilat, ilon] = NaN
                lamb_ice[imo, ilat, ilon] = NaN
                diff_ice[imo, ilat, ilon] = NaN
            else:
                spec_ice[imo, ilat, ilon] = spec_month[imo, ilat, ilon]
                lamb_ice[imo, ilat, ilon] = lamb_month[imo, ilat, ilon]
                diff_ice[imo, ilat, ilon] = diff_month[imo, ilat, ilon]
            spec_month[imo, ilat, ilon] = mean(emis_spec[ilat, ilon, :][nonzero(isnan(emis_spec[ilat, ilon, :]) == False)])
            lamb_month[imo, ilat, ilon] = mean(emis_lamb[ilat, ilon, :][nonzero(isnan(emis_lamb[ilat, ilon, :]) == False)])
            diff_month[imo, ilat, ilon] = mean(emis_diff[ilat, ilon, :][nonzero(isnan(emis_diff[ilat, ilon, :]) == False)])
            Ss = sum((emis_spec[ilat, ilon, 0 : month_day[imo]][nonzero(isnan(emis_spec[ilat, ilon, 0 : month_day[imo]]) == False)] - spec_month[imo, ilat, ilon])**2)
            std_spec[imo, ilat, ilon] = sqrt((1. / month_day[imo] - 1.) * Ss)
            Sl = sum((emis_lamb[ilat, ilon, 0 : month_day[imo]][nonzero(isnan(emis_lamb[ilat, ilon, 0 : month_day[imo]]) == False)] - lamb_month[imo, ilat, ilon])**2)
            std_lamb[imo, ilat, ilon] = sqrt((1. / month_day[imo] - 1.) * Sl)
    



# test:
ion()
x_ind, y_ind, z_ind, volume = arctic_map.arctic_map_lat50()
x_coast = x_ind
y_coast = y_ind
z_coast = z_ind
for imo in range (0, M):
    map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, grad_ratio[imo, :, :], grad_ratio[imo,:,:][nonzero(isnan(grad_ratio[imo,:,:]) == False)].min(), grad_ratio[imo, :, :][nonzero(isnan(grad_ratio[imo , :, :]) == False)].max(), 0.001, cm.s3pcpn_l_r, 'Gradient ratio [23.8GHz - 89GHz]')
    title('AMSUA ' + str(frequ) + ' - ' + str(month[imo]) + ' 2009')
    plt.savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/ice_class_AMSUA/sub_classification/maps_sea_ice_extent/emiss_diff_anomaly_map_AMSUA'+str(frequ[ifr])+'_with_AMSUA23-and-30-spec_'+str(month[imo])+'_2009.png')
#cm.s3pcpn_l_r

imo = 1
map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, std_spec[imo, :, :], std_spec[imo,:,:][nonzero(isnan(std_spec[imo,:,:]) == False)].min(), std_spec[imo, :, :][nonzero(isnan(std_spec[imo , :, :]) == False)].max(), 0.001, cm.s3pcpn_l_r, 'std spec')
'''
