#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm
from netCDF4 import Dataset
import scipy.special
from matplotlib import colors
import arctic_map
import map_cartesian_grid




##########################
# time period parameters #
##########################
MONTH = np.array(['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'])
month = np.array(['JANUARY', 'FEBRUARY', 'MARCH', 'APRIL', 'MAY', 'JUNE', 'JULY', 'AUGUST', 'SEPTEMBER', 'OCTOBER', 'NOVEMBER', 'DECEMBER'])
month_day = np.array([31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31])
M = len(month)



########################
# grid characteristics #
########################
x0 = -3000. # min limit of grid
x1 = 2500. # max limit of grid
dx = 40. # resolution for AMSUA data = 40km // resultion for AMSUB data = 20km
xvec = np.arange(x0, x1+dx, dx)
nx = len(xvec) 
y0 = -3000. # min limit of grid
y1 = 3000. # max limit of grid
dy = 40.  # resolution for AMSUA data = 40km // resultion for AMSUB data = 20km
yvec = np.arange(y0, y1+dy, dy)
ny = len(yvec)


frequ = np.array(['23', '30', '50', '89'])
for ifr in range (3, 4):
    print 'frequency ' + str(frequ[ifr]) + 'GHz'
    ###################
    # Read AMSUA data #
    ###################
    es_month = np.zeros([M, ny, nx], float)
    el_month = np.zeros([M, ny, nx], float)
    esl_month = np.zeros([M, ny, nx], float)
    std_es_month = np.zeros([M, ny, nx], float)
    std_el_month = np.zeros([M, ny, nx], float)
    std_esl_month = np.zeros([M, ny, nx], float)
    for imo in range (0, M):
        print 'open file ' + str(month[imo])
        fichier = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/monthly_GLACE/gridded_data/cartesian_grid/res_40/cartesian_grid_monthly_data_lamb_spec_near_nadir_AMSUB' + str(frequ[ifr]) + '_' + month[imo] + '2009.nc', 'r', format='NETCDF3_CLASSIC')
        lon = fichier.variables['longitude'][:]
        lat = fichier.variables['latitude'][:]
        days = fichier.variables['days'][:]
        es = fichier.variables['e_spec'][:]
        el = fichier.variables['e_lamb'][:]
        esl = fichier.variables['e_spec_lamb'][:]
        fichier.close()
        print 'compute monthly mean and std of data for mapping'
        for ilon in range (0, len(lon)):
            for ilat in range (0, len(lat)):
                es_month[imo, ilat, ilon] = mean(es[ilat, ilon, :][nonzero(isnan(es[ilat, ilon, :]) == False)])
                el_month[imo, ilat, ilon] = mean(el[ilat, ilon, :][nonzero(isnan(el[ilat, ilon, :]) == False)]) 
                esl_month[imo, ilat, ilon] = mean(esl[ilat, ilon, :][nonzero(isnan(esl[ilat, ilon, :]) == False)])
                std_es_month[imo, ilat, ilon] = sqrt((1. / (month_day[imo] - 1)) * sum((es[ilat, ilon, :][nonzero(isnan(es[ilat, ilon, :]) == False)] - es_month[imo, ilat, ilon])**2))
                std_el_month[imo, ilat, ilon] = sqrt((1. / (month_day[imo] - 1)) * sum((el[ilat, ilon, :][nonzero(isnan(el[ilat, ilon, :]) == False)] - el_month[imo, ilat, ilon])**2))
                std_esl_month[imo, ilat, ilon] = sqrt((1. / (month_day[imo] - 1)) * sum((esl[ilat, ilon, :][nonzero(isnan(esl[ilat, ilon, :]) == False)] - esl_month[imo, ilat, ilon])**2))
    ##################################
    # stack std data in netcdf files #
    ##################################
    print 'start stacking'
    for imo in range (0, M):
        print 'stack in file month ' + str(month[imo])
        rootgrp = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/monthly_GLACE/gridded_data/cartesian_grid/res_40/cartesian_grid_monthly_mean-std_data_lamb_spec_near_nadir_AMSUB' + str(frequ[ifr]) + '_' + month[imo] + '2009.nc', 'w', format='NETCDF3_CLASSIC')
        rootgrp.createDimension('longitude', nx)
        rootgrp.createDimension('latitude', ny)
        nc_lon = rootgrp.createVariable('longitude', 'f', ('longitude',))
        nc_lat = rootgrp.createVariable('latitude', 'f', ('latitude',))
        nc_mean_spec = rootgrp.createVariable('mean_spec', 'f', ('latitude', 'longitude'))
        nc_mean_lamb = rootgrp.createVariable('mean_lamb', 'f', ('latitude', 'longitude'))
        nc_mean_lamb_spec = rootgrp.createVariable('mean_lamb_spec', 'f', ('latitude', 'longitude'))
        nc_std_spec = rootgrp.createVariable('std_spec', 'f', ('latitude', 'longitude'))
        nc_std_lamb = rootgrp.createVariable('std_lamb', 'f', ('latitude', 'longitude'))
        nc_std_lamb_spec = rootgrp.createVariable('std_lamb_spec', 'f', ('latitude', 'longitude'))
        nc_lon[:] = xvec
        nc_lat[:] = yvec
        nc_mean_spec[:] = es_month[imo, :, :]
        nc_mean_lamb[:] = el_month[imo, :, :]
        nc_mean_lamb_spec[:] = esl_month[imo, :, :]
        nc_std_spec[:] = std_es_month[imo, :, :]
        nc_std_lamb[:] = std_el_month[imo, :, :]
        nc_std_lamb_spec[:] = std_esl_month[imo, :, :]
        rootgrp.close()
    '''
    print 'map of data'
    ion()
    x_ind, y_ind, z_ind, volume = arctic_map.arctic_map_lat50()
    x_coast = x_ind
    y_coast = y_ind
    z_coast = z_ind
    colormap = cm.s3pcpn_l_r
    for imo in range (0, M):
        print 'map month ' + month[imo]
        # emiss SPEC
        print 'emis spec'
        map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, es_a_month[imo, :, :], 0.45, 1.02, 0.01, colormap, 'emissivity SPEC')
        title(month[imo] + ' 2009 - AMSUA 30GHz')
        savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/comparison_lamb_spec/space_evolution/EMIS/cartesian_grid/emis_spec_AMSUA' + str(frequ[ifr]) + '_' + month[imo] + '2009.png')
        # emis SPEC-LAMB
        print 'emis lamb-spec'
        map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, esl_a_month[imo, :, :], 0., 0.03, 0.001, colormap, 'emissivity LAMB - SPEC')
        title(month[imo] + ' 2009 - AMSUA 30GHz')
        savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/comparison_lamb_spec/space_evolution/EMIS/cartesian_grid/emis_spec-lamb_AMSUA' + str(frequ[ifr]) + '_' + month[imo] + '2009.png')
        # std SPEC
        print 'std emis spec'
        map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, std_es_month[imo, :, :], 0., std_es_month[nonzero(isnan(std_es_month) == False)].max(), 0.001, colormap, 'std of emissivity SPEC')
        title(month[imo] + ' 2009 - AMSUA ' + str(frequ[ifr]) + 'GHz')
        savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/comparison_lamb_spec/space_evolution/EMIS/cartesian_grid/std_emis_spec_AMSUA' + str(frequ[ifr]) + '_' + month[imo] + '2009.png')
        # std LAMB
        print 'std emis lamb'
        map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, std_el_month[imo, :, :], 0., std_el_month[nonzero(isnan(std_el_month) == False)].max(), 0.001, colormap, 'std of emissivity LAMB')
        title(month[imo] + ' 2009 - AMSUA ' + str(frequ[ifr]) + 'GHz')
        savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/comparison_lamb_spec/space_evolution/EMIS/cartesian_grid/std_emis_lamb_AMSUA' + str(frequ[ifr]) + '_' + month[imo] + '2009.png')
        # std LAMB-SPEC
        print 'std emis lamb-spec'
        map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, std_esl_month[imo, :, :], 0., std_esl_month[nonzero(isnan(std_esl_month) == False)].max(), 0.0001, colormap, 'std emissivity LAMB - SPEC')
        title(month[imo] + ' 2009 - AMSUA ' + str(frequ[ifr]) + 'GHz')
        savefig('/usr/home/lahlod/twice_d/fig_output_ARCTIC/fig_output_sea_ice_study/comparison_lamb_spec/space_evolution/EMIS/cartesian_grid/std_emis_lamb-spec_AMSUA' + str(frequ[ifr]) + '_' + month[imo] + '2009.png')
        '''



