#!/usr/bin/env python
# -*- coding: utf-8 -*-
import string
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid, cm
from netCDF4 import Dataset
import scipy.special
import ffgrid2
import map_ffgrid
from matplotlib import colors
import cartesian_grid_monthly
import arctic_map
import map_cartesian_grid



##########################
# time period parameters #
##########################
MONTH = np.array(['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'])
month = np.array(['JANUARY', 'FEBRUARY', 'MARCH', 'APRIL', 'MAY', 'JUNE', 'JULY', 'AUGUST', 'SEPTEMBER', 'OCTOBER', 'NOVEMBER', 'DECEMBER'])
month_day = np.array([31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31])
M = len(month)



########################
# grid characteristics #
########################
x0 = -3000. # min limit of grid
x1 = 2500. # max limit of grid
dx = 40.
xvec = np.arange(x0, x1+dx, dx)
nx = len(xvec) 
y0 = -3000. # min limit of grid
y1 = 3000. # max limit of grid
dy = 40.
yvec = np.arange(y0, y1+dy, dy)
ny = len(yvec)



amb_ice = np.zeros([M, ny, nx, 31], float)
es_a = np.zeros([M, ny, nx, 31], float)
el_a = np.zeros([M, ny, nx, 31], float)
esl_a = np.zeros([M, ny, nx, 31], float)
es_b = np.zeros([M, ny, nx, 31], float)
el_b = np.zeros([M, ny, nx, 31], float)
esl_b = np.zeros([M, ny, nx, 31], float)
for imo in range (0, M):
    ###################
    # Read AMSUA data #
    ###################
    print 'open AMSUA23 file ' + str(month[imo])
    file_amsua23 = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/monthly_GLACE/gridded_data/cartesian_grid_monthly_data_lamb_spec_near_nadir_AMSUA23_' + month[imo] + '2009.nc', 'r', format='NETCDF3_CLASSIC')
    lon_amsua = file_amsua23.variables['longitude'][:]
    lat_amsua = file_amsua23.variables['latitude'][:]
    days_amsua = file_amsua23.variables['days'][:]
    es_a[imo, :, :, 0 : month_day[imo]] = file_amsua23.variables['e_spec'][:]
    el_a[imo, :, :, 0 : month_day[imo]] = file_amsua23.variables['e_lamb'][:, :, 0 : month_day[imo]]
    esl_a[imo, :, :, 0 : month_day[imo]] = file_amsua23.variables['e_spec_lamb'][:, :, 0 : month_day[imo]]
    file_amsua23.close()
    ###################
    # Read AMSUB data #
    ###################
    print 'open AMSUB89 file ' + str(month[imo])
    file_amsub89 = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/monthly_GLACE/gridded_data/cartesian_grid_monthly_data_lamb_spec_near_nadir_AMSUB89_' + month[imo] + '2009.nc', 'r', format='NETCDF3_CLASSIC')
    lon_amsub = file_amsub89.variables['longitude'][:]
    lat_amsub = file_amsub89.variables['latitude'][:]
    days_amsua = file_amsub89.variables['days'][:]
    es_b[imo, :, :, 0 : month_day[imo]] = file_amsub89.variables['e_spec'][:, :, 0 : month_day[imo]]
    el_b[imo, :, :, 0 : month_day[imo]] = file_amsub89.variables['e_lamb'][:, :, 0 : month_day[imo]]
    esl_b[imo, :, :, 0 : month_day[imo]] = file_amsub89.variables['e_spec_lamb'][:, :, 0 : month_day[imo]]
    file_amsub89.close()
    ####################
    # Read OSISAF data #
    ####################
    print 'open OSISAF file ' + str(month[imo])
    file_osisaf = Dataset('/net/argos/data/parvati/lahlod/ARCTIC/OSISAF_Ice_Types/OSISAF_ice_types_cartesian_grid/OSISAF_ice_types_cartes_grid_' + month[imo] + '_2009.nc', 'r', format='NETCDF3_CLASSIC')
    lon_osi = file_osisaf.variables['longitude'][:]
    lat_osi = file_osisaf.variables['latitude'][:]
    days_osi = file_osisaf.variables['days'][:]
    ice_type_osi = file_osisaf.variables['osi_ice_type'][:]
    file_osisaf.close()
    ###############
    # daily study #
    ###############
    for ijr in range (0, month_day[imo]):
        print 'day ' + str(ijr)
        for ilat in range (0, len(lat_osi)):
            for ilon in range (0, len(lon_osi)):
                if (ice_type_osi[ijr, ilat, ilon] > 3.): # select only data points located in 'ambiguous ice type' zones defined by OSISAF
                    amb_ice[imo, ilat, ilon, ijr] = 1.

                        
# select a pixel to study (0, 0, 33, 62)


#90, 51
#95, 82

spec_amsua = np.append(es_a[0, 95, 82, 0 : 31], es_a[1, 95, 82, 0 : 28])
spec_amsub = np.append(es_b[0, 95, 82, 0 : 31], es_b[1, 95, 82, 0 : 28])
ambig = np.append(amb_ice[0, 95, 82, 0 : 31], amb_ice[1, 95, 82, 0 : 28])
for imo in range (2, M):
    spec_amsua = np.append(spec_amsua, es_a[imo, 95, 82, 0 : month_day[imo]])
    spec_amsub = np.append(spec_amsub, es_b[imo, 95, 82, 0 : month_day[imo]])
    ambig = np.append(ambig, amb_ice[imo, 95, 82, 0 : month_day[imo]])


# seperately AMSUA and AMSUB
ion()
figure()
plot(np.arange(0, 365, 1), spec_amsua, 'b', label = 'AMSUA 23GHz')
plot(np.arange(0, 365, 1), spec_amsub, 'g', label = 'AMSUB 89GHz')
plot(np.arange(0, 365, 1)[nonzero(ambig == 1.)], spec_amsua[nonzero(ambig == 1.)], 'ob', label = 'ambiguous area')
plot(np.arange(0, 365, 1)[nonzero(ambig == 1.)], spec_amsub[nonzero(ambig == 1.)], 'og', label = 'ambiguous area')
legend(loc = 3)
xlim(-3, 368)
xticks(np.array([0, 31, 59, 90, 120, 151, 182, 212, 243, 273, 304, 334]), month, rotation = 20)
grid()

# bias AMSUA-AMSUB
figure()
plot(np.arange(0, 365, 1), spec_amsua - spec_amsub, 'r', label = 'AMSUA23 - AMSUB89')
plot(np.arange(0, 365, 1)[nonzero(ambig == 1.)], (spec_amsua - spec_amsub)[nonzero(ambig == 1.)], 'or', label = 'ambiguous area')
plot(np.arange(0, 365, 1), np.zeros([365], float), 'k')
legend(loc = 3)
xlim(-3, 368)
xticks(np.array([0, 31, 59, 90, 120, 151, 182, 212, 243, 273, 304, 335]), month, rotation = 20)


# anomaly with a 15 day time range 
spec_anom = np.zeros([365], float)
spec_diff = spec_amsua - spec_amsub
for ii in range (15, 365 - 15):
    spec_anom[ii] = (spec_diff[ii]) - mean(spec_diff[ii - 15 : ii + 15][nonzero(isnan(spec_diff[ii - 15 : ii + 15]) == False)])

figure()
plot(np.arange(0, 365, 1), spec_anom, 'r')
plot(np.arange(0, 365, 1)[nonzero(ambig == 1.)], spec_anom[nonzero(ambig == 1.)], 'or', label = 'ambiguous area')
plot(np.arange(0, 365, 1), np.zeros([365], float), 'k')
legend(loc = 3)
xlim(-3, 368)
xticks(np.array([0, 31, 59, 90, 120, 151, 182, 212, 243, 273, 304, 335]), month, rotation = 20)





# plot of anomaly (espec amsub - espec amsua) on a 15 day sliding base (15 day correct to simulate season transition)

colormap = cm.s3pcpn_l_r
x_ind, y_ind, z_ind, volume = arctic_map.arctic_map_lat50()
x_coast = x_ind
y_coast = y_ind
z_coast = z_ind
colormap = colors.ListedColormap(['cyan','blue'])
map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, amb_es_a[9, ijr, :, :], 0.4, 1.02, 0.01, colormap, 'test')
for imo in range (0, M):
    map_cartesian_grid.draw_map_cartes_l(x_coast, y_coast, z_coast, volume, xvec, yvec, amb_ice[imo, :, :, 5], 0, 1.5, 0.5, colormap, 'ambiguous ice zone')
    title(month[imo] + ' 2009 - OSISAF')
    plt.savefig('/usr/home/lahlod/twice_d/figure_output_ARCTIC/figure_output_CEN/OSISAF_ice_types/cartesian_grid/OSISAF_cartesian_grid_ambiguous_ice_tye_' +  month[imo] + '2009.png')


